﻿//
// NTFS - Myriadbits .NET NTFS library. 
// Fast MFT reader.
// Copyright (C) 2014 Myriadbits, Jochem Bakker
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// For more information, contact me at: info@myriadbits.com
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace Myriadbits.NTFS
{
	/// <summary>
	/// Class that represents a single file
	/// </summary>
	public class File
	{
		public static int UnknownRank = 999999;

		public ulong ReferenceNumber { get; set; }
		public ulong Size { get; set; }
		public string Name { get; set; }
		public short DOSFilePermissions { get; set; }
		public Folder Folder { get; set; }
		public int Rank { get; set; }


		/// <summary>
		/// Constructor, create a new file object
		/// </summary>
		/// <param name="folder"></param>
		/// <param name="referenceNumber"></param>
		/// <param name="size"></param>
		/// <param name="name"></param>
		public File(Folder folder, ulong referenceNumber, ulong size, string name)
		{
			this.Folder = folder;
			this.ReferenceNumber = referenceNumber;
			this.Size = size;
			this.Name = name;
			this.Rank = File.UnknownRank; // Not set
		}


		/// <summary>
		/// Return the full path of this file
		/// </summary>
		public string FullPath
		{
			get
			{
				if (this.Folder != null)
					return Path.Combine(this.Folder.FullPath, this.Name);
				return "";
			}
		}
	}
}
