﻿//
// NTFS - Myriadbits .NET NTFS library. 
// Fast MFT reader.
// Copyright (C) 2014 Myriadbits, Jochem Bakker
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Myriadbits.NTFS
{
	class NTFSAttributeFactory
	{
		public static NTFSAttribute CreateAttribute(DataBuffer buffer, int offset)
		{
			NTFSAttribute attribute = null;
			uint attributeType = buffer.GetUInt(offset + 0x00);
			switch (attributeType)
			{
				case 0x10: // $STANDARD_INFORMATION
					attribute = new NTFSAttributeStandardInformation(buffer, offset);
					break;
				case 0x20: // $ATTRIBUTE_LIST
					attribute = new NTFSAttributeAttributeList(buffer, offset);
					break;
				case 0x30: // $FILE_NAME
					attribute = new NTFSAttributeFilename(buffer, offset);
					break;
				case 0x40: // $OBJECT_ID
					attribute = new NTFSAttributeObjectId(buffer, offset);
					break;
				case 0x50: // $SECURITY_DESCRIPTION
					attribute = new NTFSAttributeSecurityDescriptor(buffer, offset);
					break;
				case 0x60: // $VOLUME_NAME
					attribute = new NTFSAttributeVolumeName(buffer, offset);
					break;
				case 0x70: // $VOLUME_INFORMATION
					attribute = new NTFSAttributeVolumeInformation(buffer, offset);
					break;
				case 0x80: // $DATA
					attribute = new NTFSAttributeData(buffer, offset);
					break;
				case 0x90: // $INDEX_ROOT
					attribute = new NTFSAttributeIndexRoot(buffer, offset);
					break;
				case 0xA0: // $INDEX_ALLOCATION
					attribute = new NTFSAttributeIndexAllocation(buffer, offset);
					break;
				case 0xB0: // $BITMAP
					attribute = new NTFSAttributeBitmap(buffer, offset);
					break;
				case 0xC0: // $REPARSE_POINT
					attribute = new NTFSAttributeReparsePoint(buffer, offset);
					break;
				case 0xD0: // $EA_INFORMATION
					attribute = new NTFSAttributeEAInformation(buffer, offset);
					break;
				case 0xE0: // $EA
					attribute = new NTFSAttributeEA(buffer, offset);
					break;
				case 0xF0: // $LOGGED_UTILITY_STREAM
					attribute = new NTFSAttributeLoggedUtilityStream(buffer, offset);
					break;
				case 0xFFFFFFFF: // END MARKER
					attribute = null;
					break;
				default:
					attribute = new NTFSAttribute(buffer, offset);
					break;
			}
			return attribute;
		}
	}
}
