﻿//
// NTFS - Myriadbits .NET NTFS library. 
// Fast MFT reader.
// Copyright (C) 2014 Myriadbits, Jochem Bakker
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// For more information, contact me at: info@myriadbits.com
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Myriadbits.NTFS
{
	public enum NTFSAttributeType
	{
		Standard_Information	= 0x10,
		Attribute_List			= 0x20,
		File_Name				= 0x30,
		Object_Id				= 0x40,
		Security_Descriptor		= 0x50,
		Volume_Name				= 0x60,
		Volume_Information		= 0x70,
		Data					= 0x80,
		Index_Root				= 0x90,
		Index_Allocation		= 0xA0,
		Bitmap					= 0xB0,
		Reparse_Point			= 0xC0,
		EA_Information			= 0xD0,
		EA						= 0xE0,
		Property_Set			= 0xF0,
		Logged_Utility_Stream	= 0x100
	}

	public class NTFSAttribute : DataBufferReader
	{
		// Constructor
		public NTFSAttribute(DataBuffer buffer, int offset)
			: base(buffer, offset, 0)
		{	
			if (this.IsResident)
				this._dataoffset = this._offset + this.Resident_AttributeOffset;
			else
				this._dataoffset = this._offset + 0; // TODO
		}

		// Attributes
		public NTFSAttributeType AttributeType { get { return (NTFSAttributeType)this._buffer.GetInt(this._offset + 0x00); } }
		public int Length { get { return this._buffer.GetInt(this._offset + 0x04) & 0xFFFF; } }
		public bool IsResident { get { return (this._buffer.GetByte(this._offset + 0x08) == 0x00); } }
		public byte NameLength { get { return this._buffer.GetByte(this._offset + 0x09); } }
		public short OffsetToTheName { get { return this._buffer.GetShort(this._offset + 0x0A); } }
		public ushort AttributeFlags { get { return this._buffer.GetUShort(this._offset + 0x0C); } }
		public short AttributeID { get { return this._buffer.GetShort(this._offset + 0x0E); } }
		public string AttributeName { get { return this._buffer.GetString(this._offset + 0x18, this.NameLength * 2); } }

		public byte NonResFlag { get { return this._buffer.GetByte(this._offset + 0x08); } }
		//
		// Resident properties
		public int Resident_AttributeLength { get { return this._buffer.GetInt(this._offset + 0x10); } }
		public short Resident_AttributeOffset { get { return this._buffer.GetShort(this._offset + 0x14); } }
		public byte Resident_IndexedFlag { get { return this._buffer.GetByte(this._offset + 0x16); } }

		//
		// Non-resident properties
		public ulong NonResident_StartingVCN { get { return this._buffer.GetULong(this._offset + 0x10); } }
		public ulong NonResident_LastVCN { get { return this._buffer.GetULong(this._offset + 0x18); } }
		public short NonResident_DatarunOffset { get { return this._buffer.GetShort(this._offset + 0x20); } }
		public int NonResident_DatarunPosition { get { return this._offset + this._buffer.GetShort(this._offset + 0x20); } }
		public short NonResident_CompressionSize { get { return this._buffer.GetShort(this._offset + 0x22); } }
		public ulong NonResident_AllocatedSize { get { return this._buffer.GetULong(this._offset + 0x28); } }
		public ulong NonResident_RealSize { get { return this._buffer.GetULong(this._offset + 0x30); } }
		public ulong NonResident_StreamSize { get { return this._buffer.GetULong(this._offset + 0x38); } }
		public string NonResident_AttributeName { get { return this._buffer.GetString(this._offset + 0x40, this.NameLength * 2); } }
		

		public override string ToString()
		{
			return string.Format("${0}: {1}", this.AttributeType.ToString(), base.ToStringChars());
		}
	}
}
